#!/usr/bin/env node
'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

require('babel-polyfill');

var program = require('commander');
var updateNotifier = require('update-notifier');

var cli = require('../lib/cli');

var pkg = require('../package.json');

updateNotifier({ pkg: pkg }).notify();

program.version(pkg.version).option('-u, --upload [server]', 'Set the url to upload lcov data too', 'http://localhost:8080').option('-s, --serve', 'Pass this option to startup a lcov-server instance').option('-d, --db [db]', 'Set the db connection', 'mongodb://localhost:32768/lcov-server').option('-p, --parser <parser>', 'Set the parser value [lcov, cobertura, golang, jacoco], defaults to lcov', 'lcov').option('-bp, --basePath <path>', 'The path that defines the base directory where the files that were covered will be located').parse(process.argv);

var parser = program.parser,
    upload = program.upload,
    serve = program.serve,
    db = program.db,
    basePath = program.basePath;


if (parser && ['lcov', 'cobertura', 'golang', 'jacoco'].indexOf(parser) === -1) {
  console.error('parser ' + parser + ' not supported'); // eslint-disable-line
  process.exit(1);
}

if (serve) {
  process.env.MONGO_URL = process.env.MONGO_URL || db;

  require('../index');
} else {
  var input = '';
  process.stdin.resume();
  process.stdin.setEncoding('utf8');
  process.stdin.on('data', function (chunk) {
    input += chunk;
  });
  process.stdin.on('end', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var response;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            try {
              response = cli({ parser: parser, input: input, url: upload, basePath: basePath });

              if (response.error) {
                console.error('coverage not sent with reason:\n ' + response.error); // eslint-disable-line
              } else {
                console.log('\n coverage sent successfully 💚 \n'); // eslint-disable-line
              }
            } catch (ex) {
              console.error('coverage could not be parsed with reason:\n ' + ex.toString()); // eslint-disable-line
            }

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));
}