'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var fs = require('fs');
var Path = require('path');
var Url = require('url');
var http = require('http');
var https = require('https');

var lcov = require('../lib/lcov');
var cobertura = require('../lib/cobertura');
var golang = require('../lib/golang');
var jacoco = require('../lib/jacoco');

var git = require('../lib/git');
var ci = require('../lib/ci');

module.exports = function cli(_ref) {
  var parser = _ref.parser,
      input = _ref.input,
      url = _ref.url,
      basePath = _ref.basePath;

  var parsedUrl = Url.parse(url);

  return new Promise(function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(resolve, reject) {
      var info, env, output, i, path, options, operation, data, req;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return git.parse();

            case 2:
              info = _context.sent;
              env = ci();
              output = {
                service_job_id: env.service_job_id,
                service_pull_request: env.service_pull_request,
                service_name: env.service_name,
                source_files: [],
                git: {
                  author_date: info.author_date,
                  author_email: info.author_email,
                  author_name: info.author_name,
                  committer_date: info.committer_date,
                  commit: env.commit || info.commit,
                  branch: env.branch || info.branch,
                  message: env.message || info.message,
                  committer_name: env.committer_name || info.committer_name,
                  committer_email: env.committer_email || info.committer_email,
                  remotes: info.remotes
                },
                run_at: new Date()
              };
              _context.t0 = parser;
              _context.next = _context.t0 === 'cobertura' ? 8 : _context.t0 === 'golang' ? 12 : _context.t0 === 'jacoco' ? 16 : 20;
              break;

            case 8:
              _context.next = 10;
              return cobertura.parse(input);

            case 10:
              output['source_files'] = _context.sent;
              return _context.abrupt('break', 24);

            case 12:
              _context.next = 14;
              return golang.parse(input);

            case 14:
              output['source_files'] = _context.sent;
              return _context.abrupt('break', 24);

            case 16:
              _context.next = 18;
              return jacoco.parse(input);

            case 18:
              output['source_files'] = _context.sent;
              return _context.abrupt('break', 24);

            case 20:
              _context.next = 22;
              return lcov.parse(input);

            case 22:
              output['source_files'] = _context.sent;
              return _context.abrupt('break', 24);

            case 24:
              i = 0;

            case 25:
              if (!(i < output['source_files'].length)) {
                _context.next = 36;
                break;
              }

              path = basePath ? Path.resolve(process.cwd(), basePath, output['source_files'][i].file) : output['source_files'][i].file;

              if (!fs.existsSync(path)) {
                _context.next = 32;
                break;
              }

              output['source_files'][i].source = fs.readFileSync(path).toString('utf8');
              output['source_files'][i].title = output['source_files'][i].file.substring(output['source_files'][i].file.lastIndexOf('/') + 1, output['source_files'][i].file.length);
              _context.next = 33;
              break;

            case 32:
              return _context.abrupt('return', reject('can not find file at ' + path));

            case 33:
              i++;
              _context.next = 25;
              break;

            case 36:
              options = {
                hostname: parsedUrl.hostname,
                port: parsedUrl.port || 80,
                path: '/api/upload',
                method: 'POST',
                headers: {
                  'Content-Type': 'application/json'
                }
              };
              operation = http;
              data = '';


              if (parsedUrl.protocol == 'https:') {
                options.port = 443;
                operation = https;
              }

              req = operation.request(options, function (res) {
                res.on('data', function (chunk) {
                  data += chunk;
                });
                res.on('end', function () {
                  try {
                    var response = JSON.parse(data);
                    if (response.error) {
                      return reject(response.error); // eslint-disable-line
                    } else {
                      return resolve(response);
                    }
                  } catch (ex) {
                    return reject(ex);
                  }
                });
              });

              req.write(JSON.stringify(output));
              req.end();

            case 43:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }());
};